@@ -283,24 +283,6 @@ let rec recurse_dict_inner f prev = function
 let recurse_dict (f : pdfobject -> pdfobject) elts =
   Dictionary (recurse_dict_inner f [] elts)
 
-(* Return a float from a PDF number. *)
-let getnum = function
-  | Real a -> a
-  | Integer a -> float a
-  | _ -> raise (PDFError "Pdf.getnum: not a number")
-
-(* Parse a PDF rectangle data structure. Returns min x, min y, max x, max y. *)
-let parse_rectangle = function
-  | Array [a; b; c; d] ->
-      begin try
-        let x, y, x', y' =
-          getnum a, getnum b, getnum c, getnum d
-        in
-          fmin x x', fmin y y', fmax x x', fmax y y'
-      with
-        PDFError _ -> raise (PDFError "Pdf.parse_rectangle: bad rectangle")
-      end
-  | _ -> raise (PDFError "Pdf.parse_rectangle: not a rectangle")
 
 let change_obj doc i obj =
   fst (pdfobjmap_find i doc.objects.pdfobjects) := Parsed obj
@@ -364,6 +346,26 @@ let resolve_all_delayed_object_streams pdf =
     (function (n, _) -> ignore (lookup_obj pdf n))
     (pdfobjmap_bindings pdf.objects.pdfobjects)
 
+(* Return a float from a PDF number. *)
+let rec getnum pdf = function
+  | Real a -> a
+  | Integer a -> float a
+  | Indirect i -> getnum pdf (lookup_obj pdf i)
+  | _ -> raise (PDFError "Pdf.getnum: not a number")
+
+(* Parse a PDF rectangle data structure. Returns min x, min y, max x, max y. *)
+let parse_rectangle pdf = function
+  | Array [a; b; c; d] ->
+      begin try
+        let x, y, x', y' =
+          getnum pdf a, getnum pdf b, getnum pdf c, getnum pdf d
+        in
+          fmin x x', fmin y y', fmax x x', fmax y y'
+      with
+        PDFError _ -> raise (PDFError "Pdf.parse_rectangle: bad rectangle")
+      end
+  | _ -> raise (PDFError "Pdf.parse_rectangle: not a rectangle")
+
 let catalog_of_pdf pdf =
   try lookup_obj pdf pdf.root with
     Not_found -> raise (PDFError "No catalog")
@@ -412,7 +414,7 @@ let lookup_direct pdf key dict =
   match direct pdf dict with
   | Dictionary d | Stream {contents = (Dictionary d, _)} ->
       begin match lookup_string_compare key d with
-      | None -> None
+      | None | Some Null -> None
       | Some o -> Some (direct pdf o)
       end
   | _ -> None
@@ -446,7 +448,7 @@ let lookup_exception (exp : exn) pdf key dict =
     | _ -> raise (PDFError ("lookup_exception: not a dictionary, key = " ^ key))
   in
     match lookup key dict' with
-    | None -> raise exp
+    | None | Some Null -> raise exp
     | Some v -> direct pdf v
 
 (* A specialised one raising PDFError. *)
@@ -458,8 +460,8 @@ let parse_matrix pdf name dict =
   match lookup_direct pdf name dict with
   | None -> Pdftransform.i_matrix
   | Some (Array [a; b; c; d; e; f]) ->
-      let a = getnum a in let b = getnum b in let c = getnum c
-      in let d = getnum d in let e = getnum e in let f = getnum f in
+      let a = getnum pdf a in let b = getnum pdf b in let c = getnum pdf c
+      in let d = getnum pdf d in let e = getnum pdf e in let f = getnum pdf f in
         {Pdftransform.a = a; Pdftransform.b = b; Pdftransform.c = c;
          Pdftransform.d = d; Pdftransform.e = e; Pdftransform.f = f}
   | _ -> raise (PDFError "Malformed matrix")
@@ -857,7 +859,11 @@ let unique_key prefix obj =
 (* Given a PDF and potential filename, calculate an MD5 string and build a
 suitable /ID entry from it. *)
 let generate_id _ path gettime =
-  let d = Digest.string (path ^ string_of_float (gettime ())) in
+  let d =
+    match Sys.getenv_opt "CAMLPDF_REPRODUCIBLE_IDS" with
+    | Some "true" -> Digest.string "camlpdf"
+    | _ -> Digest.string (path ^ string_of_float (gettime ()))
+  in
     Array [String d; String d]
 
 (* Find the indirect reference given by the value associated with a key in a
