@@ -467,7 +467,7 @@ let get_raw_image pdf resources width height dict data =
     read_raw_image size colspace bpc pdf resources width height dict data
   with
     e ->
-      (*i Printf.eprintf (Pdfwrite.string_of_pdf (Pdf.direct pdf dict)); i*)
+      (*i Printf.eprintf ((Pdfwrite.string_of_pdf (Pdf.direct pdf dict)) ^ "%!"); i*)
       raise e 
 
 (* Print some debug information about an image. *)
@@ -539,7 +539,7 @@ let get_image_24bpp pdf resources stream =
                   x)) decode_entry); i*)
                     let floats =
                       match decode_entry with
-                      | Some (Pdf.Array elts) -> Array.of_list (map Pdf.getnum elts)
+                      | Some (Pdf.Array elts) -> Array.of_list (map (Pdf.getnum pdf) elts)
                       | None -> [||]
                       | _ -> raise (Pdf.PDFError "Bad /Decode")
                     in
