@@ -110,7 +110,7 @@ let merge_bookmarks changes pdfs ranges pdf =
         in
           Pdfmarks.add_bookmarks bookmarks' pdf
   with
-    e -> Printf.eprintf "failure in merge_bookmarks %s\n" (Printexc.to_string e); pdf
+    e -> Printf.eprintf "failure in merge_bookmarks %s\n%!" (Printexc.to_string e); pdf
 
 let debug_pagelabels ls =
   iter (Printf.printf "%s\n") (map Pdfpagelabels.string_of_pagelabel ls)
@@ -163,7 +163,7 @@ let rec read_name_tree pdf tree =
         if odd (length elts)
           then
              begin
-               Printf.eprintf "Bad /Names array. Name tree will be read as empty\n";
+               Printf.eprintf "Bad /Names array. Name tree will be read as empty\n%!";
                []
              end
           else pairs_of_list elts
@@ -180,7 +180,7 @@ let read_name_tree pdf tree =
       map (function (Pdf.String s, x) -> (s, x) | _ -> raise Exit) r
     with
       Exit ->
-        Printf.eprintf "Pdfmerge.read_names tree: skipping malformed name tree\n";
+        Printf.eprintf "Pdfmerge.read_names tree: skipping malformed name tree\n%!";
         []
 
 let maxsize = 10 (* Must be at least two *)
@@ -369,6 +369,33 @@ let merge_optional_content_groups pdf pdfs =
     in
       Some (Pdf.addobj pdf new_ocproperties)
 
+(* Look up any acroforms and merge their fields entries, retaining any
+   other entries from any found. This is very basic, and we need to
+   see more example files to know what to do properly. *)
+let merge_acroforms pdf pdfs =
+  let form_dicts =
+    option_map
+      (fun pdf -> Pdf.lookup_direct pdf "/AcroForm" (Pdf.catalog_of_pdf pdf))
+      pdfs
+  in
+    if form_dicts = [] then None else
+    let merged_field_arrays =
+      flatten
+        (map
+          (fun d -> match Pdf.lookup_direct pdf "/Fields" d with Some (Pdf.Array a) -> a | _ -> [])
+          form_dicts)
+    in
+    let merged_nonfield_items =
+      fold_left
+        (fun d (k, v) -> Pdf.add_dict_entry d k v)
+        (Pdf.Dictionary [])
+        (flatten (map (function (Pdf.Dictionary d) -> d | _ -> []) form_dicts))
+    in
+    let new_dict =
+      Pdf.add_dict_entry merged_nonfield_items "/Fields" (Pdf.Array merged_field_arrays)
+    in
+      Some (Pdf.addobj pdf new_dict)
+
 let merge_pdfs retain_numbering do_remove_duplicate_fonts names pdfs ranges =
   let pdfs = merge_pdfs_renumber names pdfs in
   let pdfs = merge_pdfs_rename_name_trees names pdfs in
@@ -406,6 +433,11 @@ let merge_pdfs retain_numbering do_remove_duplicate_fonts names pdfs ranges =
                 None -> extra_catalog_entries
               | Some ocgpropnum -> add "/OCProperties" (Pdf.Indirect ocgpropnum) extra_catalog_entries
             in
+            let extra_catalog_entries =
+              match merge_acroforms pdf pdfs with
+              | None -> extra_catalog_entries
+              | Some acroformnum -> add "/AcroForm" (Pdf.Indirect acroformnum) extra_catalog_entries
+            in
    let pdf = Pdfpage.add_root pagetree_num extra_catalog_entries pdf in
       (* To sort out annotations etc. *)
       let old_page_numbers =
