@@ -115,7 +115,7 @@ let string_of_pagenumber n = function
   | UppercaseRoman -> roman_upper n
   | LowercaseRoman -> roman_lower n
   | UppercaseLetters -> letter_string n
-  | LowercaseLetters -> String.lowercase (letter_string n)
+  | LowercaseLetters -> String.lowercase_ascii (letter_string n)
 
 let pagelabeltext_of_single n l =
   let realnumber =
@@ -173,10 +173,19 @@ let merge_pagelabels pdfs ranges =
       in
         coalesce (change_labels (flatten new_labels))
 
+let remove pdf =
+  let root = Pdf.lookup_obj pdf pdf.Pdf.root in
+    let rootnum =
+      Pdf.addobj pdf (Pdf.remove_dict_entry root "/PageLabels")
+    in
+      pdf.Pdf.root <- rootnum;
+      pdf.Pdf.trailerdict <-
+        Pdf.add_dict_entry pdf.Pdf.trailerdict "/Root" (Pdf.Indirect rootnum)
+
 (* For now, just a flat number tree. Doesn't check ranges are well-formed (i.e
 contiguous / nonoverlapping) *)
 let write pdf labels =
-  (*if labels <> [] then*) (* <-- removed for v2.4 *)
+  if labels = [] then remove pdf else
     let arr =
       flatten
         (map
@@ -210,12 +219,3 @@ let write pdf labels =
           pdf.Pdf.trailerdict <-
             Pdf.add_dict_entry
               pdf.Pdf.trailerdict "/Root" (Pdf.Indirect rootnum)
-
-let remove pdf =
-  let root = Pdf.lookup_obj pdf pdf.Pdf.root in
-    let rootnum =
-      Pdf.addobj pdf (Pdf.remove_dict_entry root "/PageLabels")
-    in
-      pdf.Pdf.root <- rootnum;
-      pdf.Pdf.trailerdict <-
-        Pdf.add_dict_entry pdf.Pdf.trailerdict "/Root" (Pdf.Indirect rootnum)
