@@ -1,9 +1,7 @@
-(* \chaptertitle{PDFText}{Reading and writing text} *)
+(* Reading and writing text *)
 open Pdfutil
 open Pdfio
 
-(* \section{Data type for fonts} *)
-
 (* Type 3 Specific Glyph Data *)
 type type3_glpyhs =
   {fontbbox : float * float * float * float;
@@ -31,10 +29,17 @@ type fontfile =
 type fontdescriptor =
   {ascent : float;
    descent : float;
-   leading : float;
    avgwidth : float;
    maxwidth : float;
-   fontfile : fontfile option}
+   flags : int;
+   fontbbox: float * float * float * float;
+   italicangle : float;
+   capheight : float;
+   xheight : float;
+   stemv : float;
+   fontfile : fontfile option;
+   charset : string list option;
+   tounicode : (int, string) Hashtbl.t option} (* Hack. Hide tounicode in fontdescriptor, because no space at top level *)
 
 type differences = (string * int) list
 
@@ -50,8 +55,11 @@ type encoding =
 type simple_font =
   {fonttype : simple_fonttype;
    basefont : string;
-   fontmetrics : fontmetrics option;
+   firstchar : int;
+   lastchar : int;
+   widths : int array;
    fontdescriptor : fontdescriptor option;
+   fontmetrics : fontmetrics option;
    encoding : encoding}
 
 type standard_font =
@@ -143,7 +151,7 @@ let string_of_font = function
 let read_type3_data pdf font =
   {fontbbox =
      (let obj = Pdf.lookup_fail "No fontbbox" pdf "/FontBBox" font in
-       Pdf.parse_rectangle obj);
+       Pdf.parse_rectangle pdf obj);
    fontmatrix =
      Pdf.parse_matrix pdf "/FontMatrix" font;
    charprocs = 
@@ -168,29 +176,41 @@ let read_basefont pdf font =
   | Some (Pdf.Name n) -> n
   | _ -> ""
 
+let parse_charset s =
+  let i = Pdfio.input_of_string s in
+  let gs = ref [] in
+    try
+      while true do
+        Pdfread.dropwhite i;
+        begin match Pdfread.lex_name i with
+        | Pdfgenlex.LexName s ->
+            if s = "/" then raise Exit else gs := s::!gs
+        | _ -> raise Exit
+        end
+      done;
+      []
+   with
+     _ -> rev !gs
+
 let read_fontdescriptor pdf font =
   match Pdf.lookup_direct pdf "/FontDescriptor" font with
   | None -> None
   | Some fontdescriptor ->
       let ascent =
         match Pdf.lookup_direct pdf "/Ascent" fontdescriptor with
-        | Some x -> Pdf.getnum x
+        | Some x -> Pdf.getnum pdf x
         | None -> 0.
       in let descent =
         match Pdf.lookup_direct pdf "/Descent" fontdescriptor with
-        | Some x -> Pdf.getnum x
-        | None -> 0.
-      in let leading =
-        match Pdf.lookup_direct pdf "/Leading" fontdescriptor with
-        | Some x -> Pdf.getnum x
+        | Some x -> Pdf.getnum pdf x
         | None -> 0.
       in let avgwidth =
         match Pdf.lookup_direct pdf "/AvgWidth" fontdescriptor with
-        | Some x -> Pdf.getnum x
+        | Some x -> Pdf.getnum pdf x
         | None -> 0.
       in let maxwidth =
         match Pdf.lookup_direct pdf "/MaxWidth" fontdescriptor with
-        | Some x -> Pdf.getnum x
+        | Some x -> Pdf.getnum pdf x
         | None -> 0.
       in let fontfile =
         match Pdf.find_indirect "/FontFile" fontdescriptor with
@@ -202,14 +222,25 @@ let read_fontdescriptor pdf font =
                 match Pdf.find_indirect "/FontFile3" fontdescriptor with
                 | Some i -> Some (FontFile3 i)
                 | None -> None
+      in let charset =
+        match Pdf.lookup_direct pdf "/CharSet" fontdescriptor with
+        | Some (Pdf.String s) -> Some (parse_charset s)
+        | _ -> None
       in
         Some
           {ascent = ascent;
            descent = descent;
-           leading = leading;
            avgwidth = avgwidth;
            maxwidth = maxwidth;
-           fontfile = fontfile}
+           fontfile = fontfile;
+           charset = charset;
+           flags = 0;
+           fontbbox = (0., 0., 0., 0.);
+           italicangle = 0.;
+           capheight = 0.;
+           xheight = 0.;
+           stemv = 0.;
+           tounicode = None}
 
 (* Read the widths from a font. Normally in the font descriptor, but in Type3
 fonts at the top level. *)
@@ -393,10 +424,14 @@ let read_simple_font pdf font =
       begin match simple_fonttype_of_string pdf font n with
       | Some fonttype ->
           let fontdescriptor = read_fontdescriptor pdf font in
+          let fontmetrics = read_metrics pdf font in
             SimpleFont
               {fonttype = fonttype;
                basefont = read_basefont pdf font;
-               fontmetrics = read_metrics pdf font;
+               fontmetrics;
+               firstchar = 0;
+               lastchar = 0;
+               widths = [||]; 
                fontdescriptor = fontdescriptor;
                encoding = read_encoding pdf font}
       | None -> raise (Pdf.PDFError "Not a simple font")
@@ -528,62 +563,6 @@ let read_cidkeyed_font pdf font =
   in
     CIDKeyedFont (basefont, composite_CIDfont, encoding)
 
-(* Reads a font *)
-let read_font pdf fontdict =
-  if is_standard14font pdf fontdict
-    then read_standard14font pdf fontdict
-    else if is_simple_font pdf fontdict
-      then read_simple_font pdf fontdict
-      else if is_cidkeyed_font pdf fontdict
-        then read_cidkeyed_font pdf fontdict
-        else raise (Pdf.PDFError "Unknown font type")
-
-(* Write a font. Currently only works for our own special Type 3 fonts
-generated by the Pdfcff module, but eventually, when we support font embedding,
-it'll work for lots of kinds. *)
-let write_encoding pdf = function
-  | CustomEncoding (ImplicitInFontFile, diffs) ->
-      let diffarray =
-        Pdf.Array
-          (flatten (map (function (name, number) -> [Pdf.Integer number; Pdf.Name name]) diffs))
-      in
-        let encodingdict =
-          Pdf.Dictionary
-            [("/Type", Pdf.Name "/Encoding");
-             ("/Differences", diffarray)]
-        in
-          Pdf.addobj pdf encodingdict
-  | _ -> raise (Pdf.PDFError "write_encoding: not supported")
- 
-let write_font pdf = function
-  | SimpleFont
-      {fonttype = Type3
-         {fontbbox = fontbbox;
-          fontmatrix = fontmatrix;
-          charprocs = charprocs};
-       encoding = encoding;
-       fontdescriptor = Some fontdescriptor} ->
-        let encoding_entry =
-          match encoding with
-          | ImplicitInFontFile -> []
-          | _ -> [("/Encoding", Pdf.Indirect (write_encoding pdf encoding))]
-        in
-          let dict =
-            Pdf.Dictionary
-              ([("/Type", Pdf.Name "/Font");
-               ("/Subtype", Pdf.Name "/Type3");
-               ("/FontBBox", Pdf.Array [Pdf.Real 0.; Pdf.Real 0.; Pdf.Real 0.; Pdf.Real 0.]);
-               ("/FontMatrix", Pdf.Array [Pdf.Real 0.; Pdf.Real 0.; Pdf.Real 0.; Pdf.Real 0.; Pdf.Real 0.; Pdf.Real 0.]);
-               ("/CharProcs", Pdf.Dictionary (map (fun (s, _) -> (s, Pdf.Null)) charprocs));
-               ("/FirstChar", Pdf.Integer 0);
-               ("/LastChar", Pdf.Integer 0);
-               ("/Widths", Pdf.Array [Pdf.Real 0.])] @ encoding_entry)
-          in
-            Pdf.addobj pdf dict
-  | _ -> raise (Pdf.PDFError "write_font only supports type 3 fonts")
-
-(* \section{Font encodings} *)
-
 (* Parse a /ToUnicode CMap to extract font mapping. *)
 type section =
   | BfChar of char list
@@ -736,7 +715,9 @@ let rec parse_tounicode pdf tounicode =
                 (get_sections
                    (lose Pdf.is_whitespace (charlist_of_bytes data))))
           with
-            e -> Printf.eprintf "/ToUnicode Parse Error : %s\n" (Printexc.to_string e); []
+            e ->
+              Printf.eprintf "/ToUnicode Parse Error : %s\n%!" (Printexc.to_string e);
+              raise e
           end
       | _ -> assert false
       end
@@ -745,6 +726,204 @@ let rec parse_tounicode pdf tounicode =
       parse_tounicode pdf tounicode
   | e -> raise (Pdf.PDFError (Printf.sprintf "Bad /ToUnicode %s" (Pdfwrite.string_of_pdf e)))
 
+(* We didn't originally parse the /ToUnicode when reading the font, but we
+   should have. The hack now is to hide the parsed /ToUnicode hash table in the
+   fontdescriptor. To avoid complicated code, we read the font as normal, then
+   read the /ToUnicode, and insert it into the Font descriptor. *)
+let add_tounicode pdf font fontdict =
+  let tounicode =
+    match Pdf.lookup_direct pdf "/ToUnicode" fontdict with
+    | Some tounicode ->
+        begin try
+          Some (hashtable_of_dictionary <| parse_tounicode pdf tounicode)
+        with
+          e -> Printf.eprintf "bad tounicode (%s)\n%!" (Printexc.to_string e); None
+        end
+    | None -> None
+  in
+    match font with
+    | SimpleFont ({fontdescriptor = Some fd} as r) ->
+        SimpleFont {r with fontdescriptor = Some {fd with tounicode}}
+    | SimpleFont ({fontdescriptor = None} as r) ->
+        SimpleFont {r with fontdescriptor = (* fabricate one. ok? fix somehow in future... *)
+                      Some {ascent = 0.;
+                            descent = 0.;
+                            avgwidth = 0.;
+                            maxwidth = 0.;
+                            fontfile = None;
+                            charset = None;
+                            flags = 0;
+                            fontbbox = (0., 0., 0., 0.);
+                            italicangle = 0. ;
+                            capheight = 0.;
+                            xheight = 0.;
+                            stemv = 0.;
+                            tounicode}}
+    | CIDKeyedFont (a, ({cid_fontdescriptor} as r), c) ->
+        CIDKeyedFont (a, {r with cid_fontdescriptor = {r.cid_fontdescriptor with tounicode}}, c) 
+    | x -> x 
+
+let find_tounicode = function
+  | SimpleFont {fontdescriptor = Some {tounicode}}
+  | CIDKeyedFont (_, {cid_fontdescriptor = {tounicode}}, _) -> tounicode
+  | _ -> None
+
+(* Reads a font *)
+let read_font pdf fontdict =
+  let without_tounicode =
+    if is_standard14font pdf fontdict
+      then read_standard14font pdf fontdict
+      else if is_simple_font pdf fontdict
+        then read_simple_font pdf fontdict
+        else if is_cidkeyed_font pdf fontdict
+          then read_cidkeyed_font pdf fontdict
+          else raise (Pdf.PDFError "Unknown font type")
+  in
+    add_tounicode pdf without_tounicode fontdict
+
+let make_font fontname encoding =
+  Pdf.Dictionary
+    [("/Type", Pdf.Name "/Font");
+     ("/Subtype", Pdf.Name "/Type1");
+     ("/Encoding", Pdf.Name ("/" ^ string_of_encoding encoding));
+     ("/BaseFont", Pdf.Name ("/" ^ fontname))]
+
+let write_encoding pdf = function
+  | CustomEncoding (ImplicitInFontFile, diffs) ->
+      let diffarray =
+        Pdf.Array
+          (flatten (map (function (name, number) -> [Pdf.Integer number; Pdf.Name name]) diffs))
+      in
+        let encodingdict =
+          Pdf.Dictionary
+            [("/Type", Pdf.Name "/Encoding");
+             ("/Differences", diffarray)]
+        in
+          Pdf.addobj pdf encodingdict
+  | _ -> raise (Pdf.PDFError "write_encoding: not supported")
+
+let tounicode_preamble =
+"/CIDInit /ProcSet findresource begin\n\
+12 dict begin\n\
+begincmap\n\
+/CIDSystemInfo <<\n\
+  /Registry (Adobe)\n\
+  /Ordering (UCS)\n\
+  /Supplement 0\n\
+>> def\n\
+/CMapName /Adobe-Identity-UCS def\n\
+/CMapType 2 def\n\
+1 begincodespacerange\n\
+<00><FF>\n\
+endcodespacerange\n"
+
+let tounicode_postamble =
+"endbfrange\n\
+endcmap\n\
+CMapName currentdict /CMap defineresource pop\n\
+end\n\
+end\n"
+
+let hex u =
+  let b = Buffer.create 32 in
+  String.iter (fun x -> Buffer.add_string b (Printf.sprintf "%02X" (int_of_char x))) u;
+  Buffer.contents b
+
+let tounicode_map s us =
+  let b = Buffer.create 1024 in
+  let s = ref s in
+  Buffer.add_string b tounicode_preamble;
+  Buffer.add_string b (Printf.sprintf "%i beginbfrange\n" (length us));
+  iter
+    (fun u -> Buffer.add_string b (Printf.sprintf "<%02x><%02x><%s>\n" !s !s (hex u));
+     s := !s + 1)
+    us;
+  Buffer.add_string b tounicode_postamble;
+  bytes_of_string (Buffer.contents b)
+
+(* Just for the kind produced by the font subsetter for now. *)
+let write_tounicode pdf u =
+  let bytes = tounicode_map 0 (map (fun (_, s) -> s) (sort compare ((list_of_hashtbl u)))) in
+    Pdf.addobj pdf (Pdf.Stream {contents = (Pdf.Dictionary [("/Length", Pdf.Integer (bytes_size bytes))], Pdf.Got bytes)})
+
+let write_font ?objnum pdf = function
+  | SimpleFont
+      {fonttype = Type3
+         {fontbbox = fontbbox;
+          fontmatrix = fontmatrix;
+          charprocs = charprocs};
+       encoding = encoding;
+       fontdescriptor = Some fontdescriptor} ->
+        let encoding_entry =
+          match encoding with
+          | ImplicitInFontFile -> []
+          | _ -> [("/Encoding", Pdf.Indirect (write_encoding pdf encoding))]
+        in
+          let dict =
+            Pdf.Dictionary
+              ([("/Type", Pdf.Name "/Font");
+               ("/Subtype", Pdf.Name "/Type3");
+               ("/FontBBox", Pdf.Array [Pdf.Real 0.; Pdf.Real 0.; Pdf.Real 0.; Pdf.Real 0.]);
+               ("/FontMatrix", Pdf.Array [Pdf.Real 0.; Pdf.Real 0.; Pdf.Real 0.; Pdf.Real 0.; Pdf.Real 0.; Pdf.Real 0.]);
+               ("/CharProcs", Pdf.Dictionary (map (fun (s, _) -> (s, Pdf.Null)) charprocs));
+               ("/FirstChar", Pdf.Integer 0);
+               ("/LastChar", Pdf.Integer 0);
+               ("/Widths", Pdf.Array [Pdf.Real 0.])] @ encoding_entry)
+          in
+            Pdf.addobj pdf dict
+  | SimpleFont (* This case is for TrueType fonts embedded by cpdf *)
+      {fonttype = Truetype;
+       basefont;
+       firstchar;
+       lastchar;
+       widths;
+       fontdescriptor =
+         Some {ascent; descent; avgwidth; maxwidth; flags;
+               fontbbox = (minx, miny, maxx, maxy);
+               italicangle; capheight; xheight; stemv; fontfile = Some (FontFile2 ttf);
+               tounicode};
+       encoding} ->
+      let fontdescriptor =
+        Pdf.Dictionary
+          [("/Type", Pdf.Name "/FontDescriptor");
+           ("/FontName", Pdf.Name basefont);
+           ("/Flags", Pdf.Integer flags);
+           ("/FontBBox",
+               Pdf.Array
+                 [Pdf.Integer (int_of_float minx); Pdf.Integer (int_of_float miny);
+                  Pdf.Integer (int_of_float maxx); Pdf.Integer (int_of_float maxy)]);
+           ("/ItalicAngle", Pdf.Integer (int_of_float italicangle));
+           ("/Ascent", Pdf.Integer (int_of_float ascent));
+           ("/Descent", Pdf.Integer (int_of_float descent));
+           ("/CapHeight", Pdf.Integer (int_of_float capheight));
+           ("/StemV", Pdf.Integer (int_of_float stemv));
+           ("/XHeight", Pdf.Integer (int_of_float xheight));
+           ("/AvgWidth", Pdf.Integer (int_of_float avgwidth));
+           ("/MaxWidth", Pdf.Integer (int_of_float maxwidth));
+           ("/FontFile2", Pdf.Indirect ttf)]
+      in
+      let fontdesc_num = Pdf.addobj pdf fontdescriptor in
+      let font =
+        Pdf.Dictionary
+          ([("/Type", Pdf.Name "/Font");
+           ("/Subtype", Pdf.Name "/TrueType");
+           ("/BaseFont", Pdf.Name basefont);
+           ("/FontDescriptor", Pdf.Indirect fontdesc_num);
+           ("/FirstChar", Pdf.Integer firstchar);
+           ("/LastChar", Pdf.Integer lastchar);
+           ("/Widths", Pdf.Array (map (fun i -> Pdf.Integer i) (Array.to_list widths)))]
+          @ (match tounicode with
+             | None -> [("/Encoding", Pdf.Name ("/" ^ string_of_encoding encoding))]
+             | Some u -> [("/ToUnicode", Pdf.Indirect (write_tounicode pdf u))]))
+      in
+        begin match objnum with
+        | None -> Pdf.addobj pdf font
+        | Some o -> Pdf.addobj_given_num pdf (o, font); o
+        end
+  | StandardFont (standard_font, encoding) ->
+      Pdf.addobj pdf (make_font (string_of_standard_font standard_font) encoding)
+  | _ -> raise (Pdf.PDFError "Pdftext.write_font does not support this font")
+
 (* Extracting of Text *)
 
 (* A text extractor takes a character and returns a decoded PDF codepoint, a
@@ -848,32 +1027,27 @@ parse it, we do.
 
 FIXME: InlineImages2.pdf - Acrobat can extract text, why can't we?
 *)
-let text_extractor_of_font pdf font =
+let text_extractor_of_font_real font =
   {convert =
      (let encoding =
-        match read_font pdf font with
+        match font with
         | StandardFont (_, e) -> e
         | SimpleFont {encoding = e} -> e
         | _ -> ImplicitInFontFile (* No support *)
       in
         let table = table_of_encoding encoding in
           let tounicode_table, use_tounicode =
-            match Pdf.lookup_direct pdf "/ToUnicode" font with
-            | Some tounicode ->
-                begin try
-                  hashtable_of_dictionary <| parse_tounicode pdf tounicode, true
-                with
-                  e -> Printf.eprintf "bad tounicode (%s)\n" (Printexc.to_string e); (null_hash (), false)
-                end
+            match find_tounicode font with
+            | Some tu -> tu, true
             | None -> null_hash (), false
           in
             if use_tounicode then
               function i ->
                 try
-                  begin try Hashtbl.find tounicode_table i with Not_found -> "/.notdef" end,
+                  begin try "/" ^ Hashtbl.find tounicode_table i with Not_found -> ".notdef" end,
                   codepoints_of_utf16be (Hashtbl.find tounicode_table i)
                 with
-                  _ -> (* Failed *) ("/.notdef", [i])
+                  _ -> (* Failed *) (".notdef", [i])
             else
               function i ->
                 try
@@ -882,10 +1056,14 @@ let text_extractor_of_font pdf font =
                       let codepoints = Hashtbl.find (Pdfglyphlist.glyph_hashes ()) decoded in
                         decoded, codepoints
                     with
-                      _ -> (decoded, [i])
+                      _ -> ("/" ^ decoded, [i])
                 with
-                _ -> (* Failed *) ("/.notdef", [i]));
-    font = read_font pdf font}
+                _ -> (* Failed *) (".notdef", [i]));
+    font}
+
+let text_extractor_of_font pdf fontdict =
+  let font = read_font pdf fontdict in
+    text_extractor_of_font_real font
 
 (* For now, the only composite font encoding scheme we understand is /Identity-H *)
 let is_identity_h = function
@@ -906,6 +1084,55 @@ let codepoints_of_text extractor text =
 let glyphnames_of_text extractor text =
   map fst (glyphnames_and_codepoints_of_text extractor text)
 
+(* Charcode extractor (the opposite of a text extractor) Return the character
+code for a given unicode codepoint, if it exists in this encoding and font. *)
+let charcode_extractor_of_font_real ?(debug=false) font =
+  if debug then flprint ((string_of_font font) ^ "\n");
+  let encoding =
+    match font with
+    | StandardFont (_, e) -> e
+    | SimpleFont {encoding = e} -> e
+    | _ -> ImplicitInFontFile (* No support *)
+  in
+  let tounicode_reverse_table, use_tounicode =
+    match find_tounicode font with
+    | Some parsed ->
+        if debug then Printf.printf "Found a /ToUnicode table here.\n";
+        let parsed = list_of_hashtbl parsed in
+        begin try
+            if debug then
+              iter
+                (fun (charcode, utf16be_str) -> Printf.printf "/ToUnicode entry %i --> %s\n" charcode utf16be_str)
+                parsed;
+            let reversed =
+              map (fun (charcode, s) -> (codepoints_of_utf16be s, charcode)) parsed
+            in
+            hashtable_of_dictionary reversed, true
+        with
+          e -> Printf.eprintf "bad tounicode (%s)\n%!" (Printexc.to_string e); (null_hash (), false)
+        end
+    | None -> null_hash (), false
+  in
+  let table = reverse_table_of_encoding encoding in
+  let reverse_glyph_hashes = Pdfglyphlist.reverse_glyph_hashes () in
+    function codepoint ->
+      if debug then Printf.printf "Input codepoint: %X\n" codepoint;
+      try
+        if use_tounicode then
+          Some (let r = Hashtbl.find tounicode_reverse_table [codepoint] in if debug then Printf.printf "Found charcode %i\n\n" r; r)
+        else
+          let glyphname = Hashtbl.find reverse_glyph_hashes [codepoint] in
+          if debug then Printf.printf "Found glyph name %s\n" glyphname;
+            Some (let r = Hashtbl.find table glyphname in if debug then Printf.printf "Found charcode %i\n\n" r; r)
+      with
+        Not_found ->
+          if debug then Printf.eprintf "Found no charcode for unicode codepoint %X.\n" codepoint;
+          None
+
+let charcode_extractor_of_font ?(debug=false) pdf fontdict =
+  let font = read_font pdf fontdict in
+    charcode_extractor_of_font_real ~debug font
+
 (* Is a PDF string unicode (does it have a byte order marker at the beginning). *)
 let is_unicode s =
   (String.length s >= 2) && s.[0] = '\254' && s.[1] = '\255'
@@ -919,9 +1146,9 @@ let codepoint_of_pdfdocencoding_character i =
           (Hashtbl.find Pdfglyphlist.reverse_name_to_pdf_hashes i)
       with
       | [codepoint] -> Some codepoint
-      | _ -> Printf.eprintf "codepoint_of_pdfdocencoding: bad text string (char %i)\n" i; None
+      | _ -> Printf.eprintf "codepoint_of_pdfdocencoding: bad text string (char %i)\n%!" i; None
     with
-      _ -> Printf.eprintf "codepoint_of_pdfdocencoding: bad text string (char %i)\n" i; None
+      _ -> Printf.eprintf "codepoint_of_pdfdocencoding: bad text string (char %i)\n%!" i; None
 
 (* Build a UTF-8 string from a list of unicode codepoints. *)
 let get_utf8_chars c =
@@ -951,7 +1178,7 @@ let codepoints_of_pdfdocstring s =
 
 let utf8_of_pdfdocstring s =
   try utf8_of_codepoints (codepoints_of_pdfdocstring s) with
-    e -> Printf.eprintf "utf8_of_pdfdocstring : %s\n" (Printexc.to_string e); ""
+    e -> Printf.eprintf "utf8_of_pdfdocstring : %s\n%!" (Printexc.to_string e); ""
 
 (* Build a PDFDocEncoding or UTF16BE string from a UTF8 encoded string *)
 let rec codepoints_of_utf8 = function
@@ -975,12 +1202,10 @@ let rec codepoints_of_utf8 = function
         lor ((c3 land 0b00_11_11_11) lsl 6)
         lor (c4 land 0b00_11_11_11)::codepoints_of_utf8 cs
   | _ ->
-      Printf.eprintf "Bad UTF8 in codepoints_of_utf8\n"; []
+      Printf.eprintf "Bad UTF8 in codepoints_of_utf8\n%!"; []
 
 let codepoints_of_utf8 s = codepoints_of_utf8 (map int_of_char (explode s))
 
-exception RequiresUnicode
-
 (* Looks up each codepoint in the adobe glyphmap, and look up that name in
 name_to_pdf above, raising the exception only if we find something that can't
 be handled. *)
@@ -989,28 +1214,23 @@ let rec pdfdocencoding_of_codepoints sofar = function
   | c::cs ->
        try
          pdfdocencoding_of_codepoints
-           ((Hashtbl.find Pdfglyphlist.name_to_pdf_hashes (Hashtbl.find (Pdfglyphlist.reverse_glyph_hashes ()) [c]))::sofar)
+           ((Hashtbl.find Pdfglyphlist.name_to_pdf_hashes
+               (Hashtbl.find (Pdfglyphlist.reverse_glyph_hashes ()) [c]))::sofar)
            cs
        with
-         Not_found -> raise RequiresUnicode
+         Not_found -> raise Exit
 
 let pdfdocstring_of_codepoints codepoints =
   try implode (map char_of_int (pdfdocencoding_of_codepoints [] codepoints)) with
-    RequiresUnicode -> utf16be_of_codepoints codepoints
+    Exit -> utf16be_of_codepoints codepoints
 
 let pdfdocstring_of_utf8 s =
   pdfdocstring_of_codepoints (codepoints_of_utf8 s)
 
-(** Return the character code for a given unicode codepoint, if it exists in
-this encoding. This is only really suitable for simple stuff like standard 14
-fonts, or editing text in existing fonts. *)
-let charcode_extractor_of_encoding encoding =
-  let table = reverse_table_of_encoding encoding in
-    function codepoint ->
-      try
-        let glyphname =
-          Hashtbl.find (Pdfglyphlist.reverse_glyph_hashes ()) [codepoint]
-        in
-          Some (Hashtbl.find table glyphname)
-      with
-        Not_found -> None
+(* PDF strings (except /ID in the trailer dictionary and inside page content
+   streams) are either PDFDocEncoding or UTF16BE. Many times the UTF16BE can
+   all be represented in PDFDocEncoding.  In this case, there are just lots of
+   \000 bytes getting in the way making any JSON output hard to edit. So we
+   preprocess such simple UTF16BE strings into PDFDocEncoding. *)
+let simplify_utf16be s =
+  if is_unicode s then pdfdocstring_of_utf8 (utf8_of_pdfdocstring s) else s
