@@ -14,8 +14,6 @@ type simple_fonttype =
   | Type3 of type3_glpyhs
   | Truetype
 
-type fontmetrics = float array
-
 type fontfile =
   | FontFile of int
   | FontFile2 of int
@@ -24,10 +22,17 @@ type fontfile =
 type fontdescriptor =
   {ascent : float;
    descent : float;
-   leading : float;
    avgwidth : float;
    maxwidth : float;
-   fontfile : fontfile option}
+   flags : int;
+   fontbbox: float * float * float * float;
+   italicangle : float;
+   capheight : float;
+   xheight : float;
+   stemv : float;
+   fontfile : fontfile option;
+   charset : string list option;
+   tounicode : (int, string) Hashtbl.t option}
 
 type differences = (string * int) list
 
@@ -40,11 +45,16 @@ type encoding =
   | CustomEncoding of encoding * differences
   | FillUndefinedWithStandard of encoding
 
+type fontmetrics = float array (*r widths of glyphs 0..255 *)
+
 type simple_font =
   {fonttype : simple_fonttype;
    basefont : string;
-   fontmetrics : fontmetrics option;
+   firstchar : int;
+   lastchar : int;
+   widths : int array;
    fontdescriptor : fontdescriptor option;
+   fontmetrics : fontmetrics option;
    encoding : encoding}
 
 type standard_font =
@@ -104,16 +114,25 @@ val read_font : Pdf.t -> Pdf.pdfobject -> font
 
 (** Write a font to a given document, returning the object number for the main
 font dictionary *)
-val write_font : Pdf.t -> font -> int
+val write_font : ?objnum:int -> Pdf.t -> font -> int
 
 (** {2 Utility functions} *)
 
+val is_unicode : string -> bool
+val is_identity_h : font -> bool
+
 (** A list of unicode codepoints for a UTF8 string *)
 val codepoints_of_utf8 : string -> int list
 
 (** A UTF8 string for a list of unicode codepoints *)
 val utf8_of_codepoints : int list -> string
 
+(** A list of unicode codepoints for a UTF16BE string *)
+val codepoints_of_utf16be : string -> int list
+
+(** A UTF16BE string for a list of unicode codepoints (with BOM) *)
+val utf16be_of_codepoints : int list -> string
+
 (** {2 Text from strings outside page content} *)
 
 (** Take a pdf string (which will be either pdfdocencoding or UTF16BE) and
@@ -132,6 +151,10 @@ val pdfdocstring_of_codepoints : int list -> string
 document string *)
 val codepoints_of_pdfdocstring : string -> int list
 
+(** Remake a UTF16BE string into a PDFDocEncoding string if all characters are
+    in PDFDocEncoding *)
+val simplify_utf16be : string -> string
+
 (** {2 Text from strings inside page content} *)
 
 (** The type of text extractors. *)
@@ -140,6 +163,9 @@ type text_extractor
 (** Build a text extractor from a document and font object *)
 val text_extractor_of_font : Pdf.t -> Pdf.pdfobject -> text_extractor
 
+(** Build a text extractor from a document and a font *)
+val text_extractor_of_font_real : font -> text_extractor
+
 (** Return a list of unicode points from a given extractor and string (for
 example from a [Pdfpages.Op_Tj] or [Op_TJ] operator). *)
 val codepoints_of_text : text_extractor -> string -> int list
@@ -150,9 +176,14 @@ val glyphnames_of_text : text_extractor -> string -> string list
 (** {2 Building text for strings inside page content} *)
 
 (** Return the character code for a given unicode codepoint, if it exists in
-this encoding. This is only really suitable for simple stuff like standard 14
-fonts, or editing text in existing fonts. *)
-val charcode_extractor_of_encoding : encoding -> (int -> int option)
+the encoding and font object. If [debug] is set (default false) missing
+characters are reported to stderr. *)
+val charcode_extractor_of_font : ?debug:bool -> Pdf.t -> Pdf.pdfobject -> (int -> int option)
+
+(** Return the character code for a given unicode codepoint, if it exists in
+the encoding and font. If [debug] is set (default false) missing characters are
+reported to stderr. *)
+val charcode_extractor_of_font_real : ?debug:bool -> font -> (int -> int option)
 
 (** Table of all the entries in an encoding. *)
 val table_of_encoding : encoding -> (int, string) Hashtbl.t
