@@ -50,6 +50,9 @@ let string_replace_all_lazy x x' s =
 let flprint s =
   print_string s; flush stdout
 
+let fleprint s =
+  print_string s; flush stderr
+
 (* Debug printing *)
 let dp_print = ref false
 
@@ -297,9 +300,9 @@ let i32succ = Int32.succ
 
 let i32pred = Int32.pred
 
-let i32max = Pervasives.max
+let i32max = Stdlib.max
 
-let i32min = Pervasives.min
+let i32min = Stdlib.min
 
 let i64ofi = Int64.of_int
 
@@ -331,9 +334,9 @@ let i64succ = Int64.succ
 
 let i64pred = Int64.pred
 
-let i64max = Pervasives.max
+let i64max = Stdlib.max
 
-let i64min = Pervasives.min
+let i64min = Stdlib.min
 
 let i32ofi64 = Int64.to_int32
 
@@ -1087,7 +1090,7 @@ let option_map2 f a b =
   losenones (map2 f a b)
 
 (* Integer-specialised minimum and maximum functions for speed, overriding
-\emph{Pervasives.min} and \emph{Pervasives.max}. *)
+Stdlib.min and Stdlib.max. *)
 let min (a : int) b = if a < b then a else b
 let max (a : int) b = if a > b then a else b
 
@@ -1236,7 +1239,7 @@ let rec roman n =
 
 let roman_upper = roman
 
-let roman_lower n = String.lowercase (roman n)
+let roman_lower n = String.lowercase_ascii (roman n)
 
 let memoize f =
   let result = ref None in
@@ -1244,3 +1247,34 @@ let memoize f =
       match !result with
       | Some thing -> thing
       | None -> result := Some (f ()); unopt !result
+
+let contents_of_file filename =
+  let ch = open_in_bin filename in
+    try
+      let s = really_input_string ch (in_channel_length ch) in
+        close_in ch;
+        s
+    with
+      e -> close_in ch; raise e
+
+(* A clock for debugging huge files without needing the Unix module.
+Does not work on Windows. Needs GNU version of POSIX date command (gdate
+with homebrew on MacOS). *)
+let clock () =
+  let tempfile = Filename.temp_file "cpdf" "strftime" in
+  let command = Filename.quote_command "gdate" ~stdout:tempfile ["+%S-%M-%H-%3N"] in
+  let outcode = Sys.command command in
+    if outcode > 0 then raise (Failure "Date command returned non-zero exit code") else
+      let r = contents_of_file tempfile in
+        Sys.remove tempfile;
+        let get_int o l = int_of_string (String.sub r o l) in
+            float_of_int (get_int 6 2 * 3600 + get_int 3 2 * 60 + get_int 0 2)
+         +. float_of_int (get_int 9 3) /. 1000.
+
+let time = ref 0. (*ref (clock ())*)
+
+let tt' () = ()
+  (*(*Gc.major ();*)
+  let t = clock () in
+    Printf.eprintf "Elapsed: %.2f\n%!" (t -. !time);
+    time := t*)
